////////////////////////////////////////////////////////////////////////
//
//        File  Name : pipe.cc
//        Class Name : pipes
//        Author     : Young-ki Hwang
//        Date       : April 30, 1996
//        Description: This is methods of  "pipes" class.
//                     It defines process communication between the
//                     parent and child processes.
//
///////////////////////////////////////////////////////////////////////

#include <signal.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <stdio.h>
#include <iostream.h>
#include <errno.h>
#include "pipe.h"



//
//  make_pipe()
//  Make a pipe descriptor which is used between parent and child processes.
//
pipes::make_pipe()
{
    // Make a pipe.
    if (pipe(p) < 0)
      { cout<< "Error : Can't  open the pipe\n";
        exit(1);
      }

    // Make a pipe for read non-blocking for I/O.
    if(fcntl(p[0], F_SETFL, O_NDELAY) < 0)
      { cout<< "Error : fcntl call on pipe[0] \n";
        exit(1);
      }
 
    // Make a pipe for write non-blocking for I/O.
    if(fcntl(p[1], F_SETFL, O_NDELAY) < 0)
      { cout<< "Error : fcntl call on pipe[1] \n";
        exit(1);
      }
}


//
//  receive()
//  Read a message from the pipe descritpor(p[0]).
//
int pipes::receive(char *message, char *nickname)
{   int nread;

    if((nread = read(p[0], (char *)&data, sizeof(data))) < 0 )
      { if(errno == EWOULDBLOCK) return(0);
        else 
          { cout << "Error : Can't read from the pipe\n";
            exit(1);
	  }
      }

    else
      { string1.copy(message,  data.message);
	string1.copy(nickname, data.nickname);
	return(1);
      }
}



//
// receive()
// Read a message from the pipe descritpor which is from the argument.
//
int pipes::receive(int rp, char *message, char *nickname)
{   int nread;

    if((nread = read(rp, (char *)&data, sizeof(data))) < 0 )
      { if(errno == EWOULDBLOCK) return(0);
        else 
          { cout << "Error : Can't read from the pipe\n";
            exit(1);
	  }
      }

    else
      { string1.copy(message, data.message);
	string1.copy(nickname, data.nickname);
	return(1);
      }
}


//
// send()
// write a message into the pipe descritpor.
//
int pipes::send(char *message, char *nickname)
{   int nwrite;  

    string1.copy(data.message, message);
    string1.copy(data.nickname, nickname);


    if( (nwrite = write(p[1], (char *)&data, sizeof(data))) < 0 )
      { if(errno ==  EWOULDBLOCK) return(0);
        else
          { cout << "Error : Can't write onto the pipe\n";
            exit(1);
	  } 
      }
    else return(1);
}



//
// send()
// write a message into the pipe descritpor which is from the argument.
//
int pipes::send(int wp, char *message, char *nickname)
{   int nwrite;  

    string1.copy(data.message, message);
    string1.copy(data.nickname, nickname);


    if( (nwrite = write(wp, (char *)&data, sizeof(data))) < 0 )
      { if(errno ==  EWOULDBLOCK) return(0);
        else
          { cout << "Error : Can't write onto the pipe\n";
            exit(1);
	  } 
      }
    else return(1);
    
}


// 
// send()
// write a message into the pipe descritpor(p[1]).
//
pipes::send(char *message)
{   int nwrite;  

    string1.copy(data.message, message);
    string1.copy(data.nickname, "everyone");


    if( (nwrite = write(p[1], (char *)&data, sizeof(data))) < 0 )
      { if(errno ==  EWOULDBLOCK) return(0);
        else
          { cout << "Error : Can't write onto the pipe\n";
            exit(1);
	  }
      } 
    else return(1);
   
}


// 
// send()
// write a message into the pipe descritpor which is from the argument.
//
pipes::send(int wp, char *message)
{   int nwrite;  

    string1.copy(data.message, message);
    string1.copy(data.nickname, "everyone");


    if( (nwrite = write(wp, (char *)&data, sizeof(data))) < 0 )
      { if(errno ==  EWOULDBLOCK) return(0);
        else
          { cout << "Error : Can't write onto the pipe\n";
            exit(1);
	  }
      } 
    else return(1);
}


//
//  close_writepipe()
//  Close a pipe descriptor for write(p[1]).
//
pipes::close_writepipe()
{
    close(p[1]);
}


//
//  close_writepipe()
//  Close a pipe descriptor for read(p[0]).
//
pipes::close_readpipe()
{
   close(p[0]);
}


//
//  get_readpipe()
//  Get a pipe descriptor for read(p[0]).
//
int pipes::get_readpipe()
{
   return(p[0]);
}


//
//  get_writepipe()
//  Get a pipe descriptor for write(p[1]).
//
int pipes::get_writepipe()
{
   return(p[1]);
}










